<?php

/**
 * @file
 * Drush commands related to the Only One module.
 */

/**
 * Implements hook_drush_command().
 */
function onlyone_drush_command() {
  $items['onlyone-list'] = array(
    'description' => dt('List the configured content types to have Only One node.'),
    'aliases' => array('ol'),
    'examples' => array(
      'drush onlyone-list' => dt('List the configured content types to have Only One node.'),
    ),
  );
  $items['onlyone-available'] = array(
    'description' => dt('List the available content types to have Only One node.'),
    'aliases' => array('oa'),
    'examples' => array(
      'drush onlyone-available' => dt('List the available content types to have Only One node.'),
    ),
  );
  $items['onlyone-not-available'] = array(
    'description' => dt('List the content types not available to have Only One node.'),
    'aliases' => array('ona'),
    'examples' => array(
      'onlyone-not-available' => dt('List the content types not available to have Only One node.'),
    ),
  );
  $items['onlyone-summary'] = array(
    'description' => dt('Show the status summary of all the content types.'),
    'aliases' => array('os'),
    'examples' => array(
      'onlyone-summary' => dt('Show the status summary of all the content types.'),
    ),
  );
  $items['onlyone-enable'] = array(
    'description' => dt('Enable content types to have Only One node.'),
    'arguments' => array(
      'content_type' => dt('A list of content type machine names.'),
    ),
    'aliases' => array('oen'),
    'examples' => array(
      'onlyone-enable article' => dt('Enable the content type article to have Only One node.'),
      'onlyone-enable article page' => dt('Enable the content types article and page to have Only One node.'),
    ),
  );
  $items['onlyone-disable'] = array(
    'description' => dt("Disable content types to don't have Only One node."),
    'arguments' => array(
      'content_type' => dt('A list of content type machine names.'),
    ),
    'aliases' => array('odis'),
    'examples' => array(
      'onlyone-enable article' => dt("Disable the content type article to don't have Only One node."),
      'onlyone-enable article page' => dt("Disable the content types article and page to don't have Only One node."),
    ),
  );
  return $items;
}

/**
 * Implements hook_drush_help().
 */
function onlyone_drush_help($section) {
  switch ($section) {
    case 'meta:onlyone:title':
      return dt("Only One commands");

    case 'meta:onlyone:summary':
      return dt('Interact with the Only One module functionalities.');

    case 'drush:onlyone-list':
      return dt('This command will create a list with the configured content types to have Only One node.');

    case 'drush:onlyone-available':
      return dt('This command will create a list with the available content types to have Only One node.');

    case 'drush:onlyone-not-available':
      return dt('This command will create a list with the content types not available to have Only One node.');

    case 'drush:onlyone-summary':
      return dt('This command will create a list with the status of all the content types.');

    case 'drush:onlyone-enable':
      return dt('This command will enable content types to have Only One node.');

    case 'drush:onlyone-disable':
      return dt("This command will disable content types to don't have Only One node.");
  }
}

/**
 * Callback for the onlyone-list command.
 */
function drush_onlyone_list() {
  $onlyone_content_types = variable_get('onlyone_node_types', array());

  // Cleaning the array.
  $onlyone_content_types = array_unique($onlyone_content_types);
  $index = array_search(0, $onlyone_content_types, 1);
  if ($index !== FALSE) {
    unset($onlyone_content_types[$index]);
  }

  if (count($onlyone_content_types)) {
    // Searching the content type name.
    foreach ($onlyone_content_types as $content_type) {
      $content_type_object = node_type_get_type($content_type);
      $onlyone_content_types[$content_type] = $content_type_object->name;
    }
    drush_print(dt('Content types configured to have Only One node:'));
    drush_print(' ' . implode("\n ", $onlyone_content_types));
  }
  else {
    drush_print(dt('There is not content types configured to have Only One node.'));
  }
}

/**
 * Callback for the onlyone-available.
 */
function drush_onlyone_available() {
  $available_content_types = _onlyone_available_content_types();

  if (count($available_content_types)) {
    drush_print(dt('Available content types to have Only One node:'));
    drush_print(' ' . implode("\n ", $available_content_types));
  }
  else {
    drush_print(dt('There is not available content types to have Only One node.'));
  }
}

/**
 * Callback for the onlyone-not-available command.
 */
function drush_onlyone_not_available() {
  $not_available_content_types = _onlyone_not_available_content_types();

  if (count($not_available_content_types)) {
    drush_print(dt('Not available content types to have Only One node:'));
    drush_print(' ' . implode("\n ", $not_available_content_types));
  }
  else {
    drush_print(dt('All the content types are available to have Only One node.'));
  }
}

/**
 * Callback for the onlyone-summary command.
 */
function drush_onlyone_summary() {
  $available_content_types = _onlyone_available_content_types();
  $not_available_content_types = _onlyone_not_available_content_types();
  $content_types_summary = array_merge($available_content_types, $not_available_content_types);
  sort($content_types_summary);

  drush_print(dt('Status summary of all the content types:'));
  drush_print(' ' . implode("\n ", $content_types_summary));
}

/**
 * Implements drush_hook_COMMAND_validate().
 */
function drush_onlyone_enable_validate() {
  $args = func_get_args();

  if (!_drush_onlyone_validate_content_type($args)) {
    return;
  }

  $availables_content_types = array_keys(_onlyone_available_content_types());
  $not_available_content_types = array_diff($args, $availables_content_types);

  if (count($not_available_content_types)) {
    $names = implode(', ', $not_available_content_types);
    $singular = 'The following content type are not available to have Only One node: @names';
    $plural = 'The following content types are not availables to have Only One node: @names';
    $message = drush_plural(count($not_available_content_types), $singular, $plural, array('@names' => $names));
    return drush_set_error('NON_AVAILABLE', $message);
  }
}

/**
 * Callback for the onlyone-enable command.
 */
function drush_onlyone_enable() {
  $args = array_unique(func_get_args());
  $onlyone_content_types = variable_get('onlyone_node_types', array());

  foreach ($args as $content_type) {
    $onlyone_content_types[$content_type] = $content_type;
  }

  variable_set('onlyone_node_types', $onlyone_content_types);

  $names = implode(', ', $args);
  $singular = 'The following content type was enabled to have Only One node: @names';
  $plural = 'The following content types were enabled to have Only One node: @names';
  $message = drush_plural(count($args), $singular, $plural, array('@names' => $names));
  drush_log($message, 'success');
}

/**
 * Implements drush_hook_COMMAND_validate().
 */
function drush_onlyone_disable_validate() {
  $args = func_get_args();

  if (!_drush_onlyone_validate_content_type($args)) {
    return;
  }
}

/**
 * Callback for the onlyone-disable command.
 */
function drush_onlyone_disable() {
  $args = array_unique(func_get_args());
  $onlyone_content_types = variable_get('onlyone_node_types', array());

  foreach ($args as $content_type) {
    unset($onlyone_content_types[$content_type]);
  }

  variable_set('onlyone_node_types', $onlyone_content_types);

  $names = implode(', ', $args);
  $singular = "The following content type was disabled to don't have Only One node: @names";
  $plural = "The following content types were disabled to don't have Only One node: @names";
  $message = drush_plural(count($args), $singular, $plural, array('@names' => $names));
  drush_log($message, 'success');
}

/**
 * Helper function to validate the content type names.
 *
 * @param array $content_types
 *   An array with content type machine names.
 *
 * @return bool
 *   A boolean indicating the success of the validation.
 */
function _drush_onlyone_validate_content_type($content_types) {
  if (!count($content_types)) {
    $message = dt('You must enter at least the name of one content type.');
    drush_set_error('NON_EXISTENT', $message);
    return FALSE;
  }

  $content_types_list = array_keys(node_type_get_types());
  $invalid_content_types = array_diff($content_types, $content_types_list);

  if (count($invalid_content_types)) {
    $names = implode(', ', $invalid_content_types);
    $singular = 'The following content type is invalid: @names';
    $plural = 'The following content types are invalid: @names';
    $message = drush_plural(count($invalid_content_types), $singular, $plural, array('@names' => $names));
    drush_set_error('NON_EXISTENT', $message);
    return FALSE;
  }

  return TRUE;
}

/**
 * Formats a plural string containing a count of items.
 *
 * This function ensures that the string is pluralized correctly. Since dt() is
 * called by this function, make sure not to pass already-localized strings to
 * it.
 *
 * For example:
 * @code
 *   $output = drush_plural($node->comment_count, '1 comment', '@count comments');
 * @endcode
 *
 * Example with additional replacements:
 * @code
 *   $output = drush_plural($update_count,
 *     'Changed the content type of 1 post from %old-type to %new-type.',
 *     'Changed the content type of @count posts from %old-type to %new-type.',
 *     array('%old-type' => $info->old_type, '%new-type' => $info->new_type));
 * @endcode
 *
 * @param $count
 *   The item count to display.
 * @param $singular
 *   The string for the singular case. Make sure it is clear this is singular,
 *   to ease translation (e.g. use "1 new comment" instead of "1 new"). Do not
 *   use @count in the singular string.
 * @param $plural
 *   The string for the plural case. Make sure it is clear this is plural, to
 *   ease translation. Use @count in place of the item count, as in
 *   "@count new comments".
 * @param $args
 *   An associative array of replacements to make after translation. Instances
 *   of any key in this array are replaced with the corresponding value.
 *   Based on the first character of the key, the value is escaped and/or
 *   themed. See format_string(). Note that you do not need to include @count
 *   in this array; this replacement is done automatically for the plural case.
 * @param $options
 *   An associative array of additional options. See dt() for allowed keys.
 *
 * @return
 *   A translated string.
 *
 * @see dt()
 */
function drush_plural($count, $singular, $plural, array $args = array(), array $options = array()) {
  $args['@count'] = $count;
  if ($count == 1) {
    return dt($singular, $args, $options);
  }

  // Get the plural index through the gettext formula.
  $index = (function_exists('locale_get_plural')) ? locale_get_plural($count, isset($options['langcode']) ? $options['langcode'] : NULL) : -1;
  // If the index cannot be computed, use the plural as a fallback (which
  // allows for most flexiblity with the replaceable @count value).
  if ($index < 0) {
    return dt($plural, $args, $options);
  }
  else {
    switch ($index) {
      case "0":
        return dt($singular, $args, $options);
      case "1":
        return dt($plural, $args, $options);
      default:
        unset($args['@count']);
        $args['@count[' . $index . ']'] = $count;
        return dt(strtr($plural, array('@count' => '@count[' . $index . ']')), $args, $options);
    }
  }
}
